import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import sympy as sp
from scipy import interpolate
import matplotlib.tri as tri
import matplotlib.colors as colors
from scipy import signal
import pandas as pd
import scipy.stats as stats


cm = lambda x,y : (x/2.56, y/2.56)

def to_float(ls):
    return [float(ls[i]) for i in range(len(ls))]
def to_int(ls):
    return [int(ls[i]) for i in range(len(ls))]


def make_table(df):
    
    # the data we want in the table
    indexes = ["Unnamed: 3","Unnamed: 4","Unnamed: 5","Unnamed: 6","Wing span"]

    # generate the latex file for table
    f = open("table.tex", "w")
    for i in range(1,25):
        string = "\emph{"+df.iloc[i]["Unnamed: 2"]+"} & "
        for idx in indexes:
            string += str(df.iloc[i][idx])+" & "
        string += str(df.iloc[i]["Speed range"])+"-"+str(df.iloc[i]["Unnamed: 11"])+" & "
        string += str(df.iloc[i]["Wingbeat frequency"])+"$\pm$"+str(df.iloc[i]["Unnamed: 13"])+" & "
        string += str(df.iloc[i]["Wingbeat amplitude"])+"$\pm$"+str(df.iloc[i]["Unnamed: 15"])+" & "
        string += str(df.iloc[i]["Strouhal number"])+"-"+str(df.iloc[i]["Unnamed: 17"])+"\\"+"\\"+"\n"
        f.write(string)
    f.close()
    return None


def make_axis_nice(ax):
    ax.set_xticks(np.array([0.25,0.35,1.0]))
    ax.tick_params(axis='both', which='major', labelsize=10)
    ax.tick_params(axis='both', which='minor', labelsize=10)
    # Only show ticks on the left and bottom spines
    for spine in ax.spines:
        ax.spines[spine].set_visible(False)
    ax.yaxis.set_ticks_position('left')
    ax.xaxis.set_ticks_position('bottom')
   
    ax.set_xlabel(r"Strouhal number", fontsize=10)
    ax.tick_params(axis='y',labelrotation=0)
    return ax


def compute_principal(vector):
    Exx = vector[0]; Eyy = vector[1]; Ezz = vector[2]
    Exy = vector[3]; Eyz = vector[4]; Exz = vector[5]
    tensor = np.array([[Exx, Exy, Exz],
                       [Exy, Eyy, Eyz],
                       [Exz, Eyz, Ezz]])
    
    # Calculate principal stresses for current node
    lam, phi = np.linalg.eigh(tensor)
    modeind = np.argsort(lam)[::-1]
    lam = lam[modeind]
    phi = phi[:,modeind]

    return lam, phi


def get_fibre_direction(ax):
    
    # get element names
    f = open("data/Bat_elements.nam")
    elements=[]; node=[]
    for line in f:
        if line[7:13]=="ELSET=":
            el = line[13:].strip()
            continue
        for item in line.strip().split():
            elements.append(el)
            node.append(int(item.replace(",","")))
    f.close()
    elements = np.array(elements)
    node = np.array(node)
    idx = np.argwhere(elements!='BONES')

    # get elements
    f = open("data/Bat_geom.inp")
    lines = f.readlines()[1:]
    ids=[]; x=[]; y=[]; z=[]
    for k in range(len(lines)):
        if lines[k].strip()[0:3]=="*EL":
            finish = k; break
    f.close()
    conectiv = []
    for k in range(0,finish):
        ids_,x_,y_,z_ = to_float(lines[k].strip().replace(",","").split())
        ids.append(ids_)
        x.append(x_)
        y.append(y_)
        z.append(z_)
    for k in range(finish+1,len(lines)):
        n,a,b,c = to_int(lines[k].strip().replace(",","").split())
        conectiv.append([a-2,b-2,c-2])
    conectiv = np.array(conectiv)
    x = np.array(x)
    y = np.array(y)
    z = np.array(z)

    for el,col in zip(["MEMBRANE1","MEMBRANE2"],
                      ["#454444","#666666"]):
        mask = elements!=el
        
        trian_mask = tri.Triangulation(x, y, triangles=conectiv, mask=mask)
        
        # inner membrane
        if el == "MEMBRANE1":
            alpha = 22.5
        else:
            alpha = 45.0

        for a1,a2 in zip([np.cos(np.radians(alpha)),np.cos(np.radians(alpha+90))],
                        [np.sin(np.radians(alpha)),np.sin(np.radians(alpha+90))]):
            
            # make grid
            x_min, x_max = min(trian_mask.x),max(trian_mask.x)
            y_min, y_max = min(trian_mask.y),max(trian_mask.y)
            grid_xx, grid_yy = np.mgrid[x_min:x_max:32j, y_min:1.1*y_max:32j]

            # make interpolation function
            interpolator11 = tri.CubicTriInterpolator(trian_mask, a1*np.ones_like(x), kind='min_E')
            interpolator22 = tri.CubicTriInterpolator(trian_mask, a2*np.ones_like(x), kind='min_E')
            
            # interpolate
            e1 = interpolator11(grid_xx, grid_yy)
            e2 = interpolator22(grid_xx, grid_yy)

            
    return ax, elements

# point makin ghte trailing edge
TE_point_1 = np.array([8844,2505,2460,2217,2026,1911,1870,1779,2458,
                     2119,2280,1681,1612,1600,1609,1548,1549,1633,
                     2178,2179,1693,931,1695,1696,1773,1855,1914,
                     1987,2064,2224,2464,2504,8652],dtype=int)

TE_point_2 = np.array([9228,8460,8268,8076,7884,7692,7500,7308,7116,
                     6924,6732,6540,6348,6156,5964,5772,5580,5388,
                     5196,5004,4812,4620,4428,4236,4044,3852,3660,
                     3468,3276,3084,2892,2700,8849],dtype=int)
